/**
 * Anthony VH - November 2013
 * Transmitter for the wireless quiz buttons.
 *
 * Using LowPower library to put Arduino in sleep mode.
 * See: https://github.com/rocketscream/Low-Power
 *
 * Using nRF24L01 library for RF interface.
 * See: https://github.com/aaronds/arduino-nrf24l01
 *
 * Pins:
 * Hardware SPI:
 * MISO -> 12
 * MOSI -> 11
 * SCK -> 13
 *
 * Configurable:
 * CE -> 9
 * CSN -> 8
 */

#include <SPI.h>
#include <Mirf.h>
#include <nRF24L01.h>
#include <MirfHardwareSpiDriver.h>

#include <LowPower.h>

volatile bool do_transmission;
byte buttonNr;

void setup(){
  // Setup button nr
  setButtonNr();
  char receiverAddr[] = "button0";
  receiverAddr[6] = '1' + buttonNr;

  // Setup MIRF board
  Mirf.cePin = 9;
  Mirf.csnPin = 8;

  Mirf.spi = &MirfHardwareSpi;
  Mirf.init();

  Mirf.setRADDR((byte *) receiverAddr);

  Mirf.payload = sizeof(byte);

  // we use channel 90 as it is outside of WLAN bands
  // or channels used by wireless surveillance cameras
  Mirf.channel = 90;

  Mirf.config();

  // Setup button
  pinMode(2, INPUT);           // set pin to input
  digitalWrite(2, HIGH);       // turn on pullup resistors

  // Setup other variables
  do_transmission = false;

  // Reduce power
  pinsLow();
}

void loop()
{
  if (do_transmission == true)
  {
    // Clear flag
    do_transmission = false;

    // Setup packet data
    Mirf.setTADDR((byte *) "receiver");

    // Send data
    Mirf.send((byte *) & buttonNr);

    // Wait for sending to finish
    while (Mirf.isSending())
    {
    }

    // Power down the MIRF module
    Mirf.powerDown();

    // Delay a bit, so button can't be mashed like crazy
    delay(50);
  } else
  {
    // Wait for pin 2 to go high
    while (!digitalRead(2))
    {
      delay(50);
    }

    // Go to sleep
    sleepNow();
  }
}

void wakeUpNow ()
{
  // Don't put detach here, seems to lock up chip,
  // i.e. on button bounce while trying to go into sleep.
  // Detach the interrupt so it doesn't fire again
  //detachInterrupt(0);

  // Set flag
  do_transmission = true;
}

void setButtonNr ()
{
  // Read out button number
  buttonNr = 0;
  uint8_t const pinNrs[] = {5, 4, 3};

  for (int i = 0; i < sizeof(pinNrs); ++i)
  {
    // Setup pin
    pinMode(pinNrs[i], INPUT_PULLUP);

    // Read value
    uint8_t curState = digitalRead(pinNrs[i]) ? 0 : 1;
    buttonNr += (curState << i);

    // Make pin consume low power
    pinMode(pinNrs[i], OUTPUT);
    digitalWrite(pinNrs[i], LOW);
  }
}

void pinsLow ()
{
  // Make all unused pins low outputs to save power
  uint8_t const pinNrs[] = {0, 1, 6, 7, 10, A0, A1, A2, A3, A4, A5, A6, A7};

  for (int i = 0; i < sizeof(pinNrs); ++i)
  {
    pinMode(pinNrs[i], OUTPUT);
    digitalWrite(pinNrs[i], LOW);
  }
}

void sleepNow ()         // here we put the arduino to sleep
{
  // Make sure the nRF24L01+ module is powered down
  Mirf.powerDown();

  // Make sure we don't go sleeping with pin low!
  while (!digitalRead(2))
  {
  }

  /* Now it is time to enable an interrupt. We do it here so an
   * accidentally pushed interrupt button doesn't interrupt
   * our running program. if you want to be able to run
   * interrupt code besides the sleep function, place it in
   * setup() for example.
   *
   * In the function call attachInterrupt(A, B, C)
   * A   can be either 0 or 1 for interrupts on pin 2 or 3.
   *
   * B   Name of a function you want to execute at interrupt for A.
   *
   * C   Trigger mode of the interrupt pin. can be:
   *             LOW        a low level triggers
   *             CHANGE     a change in level triggers
   *             RISING     a rising edge of a level triggers
   *             FALLING    a falling edge of a level triggers
   *
   * In all but the IDLE sleep modes only LOW can be used.
   */
  attachInterrupt(0, wakeUpNow, LOW); // use interrupt 0 (pin 2) and run function

  // Sleep with ADC module and BOD module off
  LowPower.powerDown(SLEEP_FOREVER, ADC_OFF, BOD_OFF);

  // THE PROGRAM CONTINUES FROM HERE AFTER WAKING UP

  // Detach the interrupt so it doesn't fire again
  detachInterrupt(0);
}
